#!/usr/bin/python
'''
Copyright 2009, Mikko Talkara

    This file is part of Uppaal Reader.

    Uppaal Reader is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Uppaal Reader is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Uppaal Reader.  If not, see <http://www.gnu.org/licenses/>.
'''

import re
import Queue
from xml.sax import ContentHandler

class ModelHandler(ContentHandler):
    """ModelHandler class implements ContentHandler, an interface for
handling XML document parsing events. ModelHandler parses .xml model file for
interested information, namely location and transition data and comments
associated with them."""
    def __init__(self, filename):
        """filename - name of the output file"""
        self.f = open(filename, 'w')
        self.output = ""
        self.name = ""
        self.name_dict = {}                         # dictionary for location id alias names
        self.count = 0
        self.queue = Queue.Queue(0)
        self.isCommentElement = 0
        self.isNameElement = 0
        self.isSysElement = 0
        """automation instance dictionary, where name of the instance acts as a
        value and automation class as a key"""
        self.aut_dict = {}
        
    
    def startElement(self, name, attrs):
        """method handles starting occurrences of specified elements"""
        if name == 'declaration':
            self.template = self.name
        elif name == 'location':
            self.id = attrs.get('id', "")
            self.name = ""
        elif name == 'label' and attrs.get('kind', "") == 'comments':
            self.isCommentElement = 1
            self.comment = ""
        elif name == 'source':
            self.source = attrs.get('ref')
        elif name == 'target':
            self.target = attrs.get('ref')
        elif name == 'name':
            self.isNameElement = 1
            self.name = ""
        elif name == 'system':
            self.isSysElement = 1
            self.system = ""
        return

    def endElement(self, name):
        """method handles ending occurrences of specified elements"""
        if name == 'location':
            # add new id alias name to dictionary
            self.name_dict[self.id] = self.name
            
            self.output += 'Location: ' + self.template + '.' + self.name + ' (Id=' + self.id + ')\n'
            while not (self.queue.empty()):
                string = self.queue.get()
                self.output += string + '\n'
                
        elif name == 'transition':
            """Adds
            Transition: (Name (id_source) -> id_target)
            OR
            Transition: (id_source -> id_target)
            """
            self.output += 'Transition: ('
            if self.name_dict.has_key(self.source) and \
            self.name_dict[self.source] != '':
                self.output += (self.template + '.' + self.name_dict[self.source] + 
                             ' (' + self.source + ') -> ')
            else:
                self.output += self.source + ' -> '
            if self.name_dict.has_key(self.target) and \
            self.name_dict[self.target] != '':
                self.output += (self.template + '.' + self.name_dict[self.target] + 
                             ' (' + self.target + '))\n')
            else:
                self.output += self.target + ')\n'
                
            while not (self.queue.empty()):
                string = self.queue.get()
                self.output += string + '\n'
                
        elif name == 'label' and self.isCommentElement == 1 \
        and 'State.c' in self.comment:
            self.queue.put(self.comment)
            self.isCommentElement = 0
            
        elif name == 'name':
            self.isNameElement = 0    
        elif name == 'system':
            self.isSysElement = 0

    def characters(self, ch):
        """reads data content of specified elements one character at a 
        time"""
        if self.isCommentElement == 1:
            self.comment += ch
        if self.isNameElement == 1:
            self.name += ch
        if self.isSysElement == 1:
            self.system += ch
            
    def endDocument(self):
        # automation system dict
        self.system = self.system.replace(" ", "")
        tmp_list = re.findall('\w+=\w+(?=\(\);)', self.system)
        for item in tmp_list:
            k, v = item.split('=')
            self.aut_dict[k] = v
            
        # replace automation class names with instance names
        for k in self.aut_dict.keys():
            self.output = self.output.replace(self.aut_dict[k] + '.', k + '.')
        
        """method handles document end event"""
        self.f.write(self.output)
        self.f.close()
