#!/usr/bin/python

'''
Created on Jul 2, 2009

Copyright 2009, Mikko Talkara

    This file is part of Uppaal Reader.

    Uppaal Reader is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Uppaal Reader is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Uppaal Reader.  If not, see <http://www.gnu.org/licenses/>.
'''
import wx
import os, sys
import TraceParser
import XmlParser
from xml.sax import make_parser
from xml.sax import SAXParseException
from traceback import format_exc

ID_OPENFILE = 101
ID_SAVE = 102
ID_SAVEAS = 103
ID_EXIT = 104
ID_ABOUT = 105

class GUI(wx.Frame):
    '''Class offers Graphical User Interface for Uppaal Parser'''
    def __init__(self, parent, id, title):
        '''Initializes and shows GUI'''
        
        wx.Frame.__init__(self, parent, id, title, size=(800, 600))
        
        self.save_filename = ''
        self.tmp_filename = 'tmp_file.txt'
        self.output = ''
        self.xml_file = ''
        panel = wx.Panel(self, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)
        
        # create temporary file for XML parsing
        tmp_file = open(self.tmp_filename, 'w')
        tmp_file.close()
        
        # setting up menubar
        menubar = wx.MenuBar()
        
        # file menu
        fileMenu = wx.Menu()
        fileMenu.Append(ID_OPENFILE, '&Open File', 'Open file')
        fileMenu.Append(ID_SAVE, '&Save\tCtrl+S', 'Save file')
        fileMenu.Append(ID_SAVEAS, 'Save &As...', 'Save file as')
        fileMenu.AppendSeparator()
        fileMenu.Append(ID_EXIT, '&Quit\tCtrl+Q', 'Quit application')
        fileMenu.Detach()
        
        # help menu
        helpMenu = wx.Menu()
        helpMenu.Append(ID_ABOUT, '&About Uppaal Trace Reader', 'About program')
        
        # append menus to menubar
        menubar.Append(fileMenu, '&File')
        menubar.Append(helpMenu, '&Help')
        
        self.SetMenuBar(menubar)
        
        # binding methods to events
        self.Bind(wx.EVT_MENU, self.OnOpenFile, id=ID_OPENFILE)
        self.Bind(wx.EVT_MENU, self.OnSaveAs, id=ID_SAVEAS)
        self.Bind(wx.EVT_MENU, self.OnSave, id=ID_SAVE)
        self.Bind(wx.EVT_MENU, self.OnQuit, id=ID_EXIT)
        self.Bind(wx.EVT_MENU, self.OnAbout, id=ID_ABOUT)
        
        # setting up wx.TextCtrl object (output container)
        self.label1 = wx.TextCtrl(panel, -1, self.output, style=wx.TE_READONLY|wx.TE_MULTILINE)
        sizer.Add(self.label1, 1, flag=wx.EXPAND|wx.ALL, border=10)
        panel.SetSizer(sizer)
        
        # showing gui
        self.Centre()
        self.Show(True)
    
    def OnOpenFile(self, event):
        '''Handles Open File event. 
        
        The first file chosen is meant to be an Uppaal trace 
        file to be parsed.

        The second, optional model file is an Uppaal XML file from which fetching additional comments 
        concerning transitions occurring in trace file.'''
        
        # reset filename used for saving
        self.save_filename = ''
        
        # show trace file dialog
        dlg = wx.FileDialog(self, "Choose a trace file", os.getcwd(), "", "*.*", wx.FD_OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.input_file = dlg.GetPath()
            dlg.Destroy()
     
            # show comments dialog
            dlg2 = wx.MessageDialog(None, 'Show transition comments from model file?', 'Show comments?', 
                wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
            answer = dlg2.ShowModal()
            if answer == wx.ID_YES:
                dlg2.Destroy()
                
                # show choose model file dialog
                dlg3 = wx.FileDialog(self, "Choose a model file for comments", os.getcwd(), "", "*.xml", wx.FD_OPEN)
                if dlg3.ShowModal() == wx.ID_OK:
                    self.xml_file = dlg3.GetPath()
                dlg3.Destroy()
                
                # parsing model file
                if self.xml_file:             
                    parser = make_parser()
                    curlHandler = XmlParser.ModelHandler(self.tmp_filename)
                    parser.setContentHandler(curlHandler)
                    try:
                        parser.parse(open(self.xml_file))
                    except SAXParseException:
                        # show error dialog
                        dlg4 = wx.MessageDialog(self,
                                                'Invalid model file. Make sure that you have chosen valid XML file.\n\n' \
                                                + format_exc(),
                                                'Error dialog',
                                                wx.OK|wx.ICON_ERROR)
                        dlg4.ShowModal()
                        dlg4.Destroy()
            
            elif answer == wx.ID_CANCEL:
                dlg2.Destroy()
            elif answer == wx.ID_NO:
                self.tmp_file = ''
                dlg2.Destroy()
            
            if answer != wx.ID_CANCEL:
                # parse uppaal trace file
                t_parser = TraceParser.Parser(self.input_file, self.tmp_filename)
                self.output = t_parser.parse()
                
                # refresh wx.TextCtrl object
                self.label1.Clear()
                self.label1.WriteText(self.output)

        else:
            dlg.Destroy()
    
    def OnSave(self, event, test_filename=''):
        """Handles Save event. Third parameter, test_filename, is meant for
        testing purposes only."""
        if test_filename != '':
            self.save_filename = test_filename
             
        if self.save_filename != '':
            self.file = open(self.save_filename, 'w')
            self.file.write(self.output)
            self.file.close()
        else:
            GUI.OnSaveAs(self, event)
    
    def OnSaveAs(self, event):
        '''Handles Save As... event'''
        dlg = wx.FileDialog(self, "Choose a file name", os.getcwd(), "", "*.*",
                            wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        if dlg.ShowModal() == wx.ID_OK:
            self.save_filename = dlg.GetPath()
            self.file = open(self.save_filename, 'w')
            self.file.write(self.output)
            self.file.close()
    
    def OnQuit(self, event):
        '''Handles Quit event'''
        self.Close()
        
    def OnAbout(self, event):
        '''Handels About event'''
        description = """Uppaal Reader is a program for systematic analyzing of Uppaal output files. Main feature is 
        parsing Uppaal trace and model files."""
        
        info = wx.AboutDialogInfo()
        
        info.SetName('Uppaal Reader')
        info.SetVersion('0.1')
        info.SetDescription(description)
        info.SetCopyright('(C) 2008 mikko talkara')
        
        wx.AboutBox(info)
    
    def GetOutput(self):
        """Method returns current output. Used on testing purposes."""
        return self.output

def main():
    app = wx.App()
    GUI(None, -1, 'Uppaal Trace Reader')
    app.MainLoop()

if __name__ == '__main__':
    main()
